/* $Id: ArkImage.cpp,v 1.14 2002/10/11 01:10:02 zongo Exp $
**
** Ark - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2002 The Contributors of the Ark Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <Ark/ArkImage.h>
#include <Ark/ArkSystem.h>
#include <Ark/ArkConfig.h>
#include <Ark/ArkFileSys.h>

#include <sstream>
#include <string>


namespace Ark
{

Image::Image (const String &name, int w, int h, Format format, uchar *data) :
  Object (name)
{
  m_Data = NULL;
  m_Type = V_IMAGE;

  SetFormat (w, h, format, data);
}

Image::Image () : Object ("")
{
  m_Data = NULL;
  m_Format = GRAY_8;
  m_Width = m_Height = 0;
  m_Type = V_IMAGE;
}

Image::~Image ()
{
  if (m_Data)
    delete m_Data;
}

/// Set the image format, and image data if \c data isn't null.
bool
Image::SetFormat (int w, int h, Format format, uchar *data)
{
  m_Width = w;
  m_Height = h;
  m_Format = format;

  int dsize = m_Width * m_Height;
  switch (format)
  {
    case RGB_888: dsize *= 3; break;
    case RGBA_8888: dsize *= 4; break;
    case GRAY_8: break;
  }

  if (m_Data != NULL)
    delete[] m_Data;

  m_Data = new uchar[dsize];

  if (m_Data == NULL)
    return false;

  if (data)
    memcpy (m_Data, data, dsize);
  else
    memset (m_Data, 0, dsize);

  return true;
}

void WriteTGA_RGB (WriteStream &file, uchar *image, int width, int height);
void WriteTGA_Gray (WriteStream &file, uchar *image, int width, int height);

void
Image::SaveTGA (const Ark::String &name)
{
  if (m_Format == RGBA_8888)
    return;

  FileWriteStream file(Sys()->FS()->GetFileName(name).c_str());

  if (file.bad())
    return;

  if (m_Format == RGB_888)
    WriteTGA_RGB (file, m_Data, m_Width, m_Height);
  else if (m_Format == GRAY_8)
    WriteTGA_Gray (file, m_Data, m_Width, m_Height);
}

const char *FormatToString (int format)
{
   switch (format)
   {
      case Image::RGB_888:
	 return "RGB_888";
      case Image::RGBA_8888:
	 return "RGBA_8888";
      case Image::GRAY_8:
	 return "GRAY_8";
   }

   return "UNKNOWN";
}

/// Returns a string describing the object
String
Image::Dump (bool long_version)
{
    std::ostringstream os;

    os << "[img] " << m_Name;
    
    if (long_version)
    {
	os << std::endl << "\twidth : " << m_Width;
	os << std::endl << "\theight : " << m_Height;
	os << std::endl << "\tformat : " << FormatToString (m_Format);
    }

    return os.str();
}

/* namespace Ark */
}
