/* $Id: ArkSkin.cpp,v 1.7 2002/10/14 05:13:29 mrq Exp $
**
** Ark - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the Ark Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <Ark/ArkSkin.h>
#include <Ark/ArkCache.h>
#include <Ark/ArkSystem.h>

namespace Ark
{
   /// Create an empty skin with the given name.
   Skin::Skin (const String &name) : 
      Object (name, V_SKIN)
   {}

   /// Destroy the given skin.
   Skin::~Skin ()
   {
       std::vector<Material*>::iterator it;

      for (it = m_Materials.begin(); it != m_Materials.end(); it++)
	 (*it)->Unref();
   }

   /// Load the skin from the given file.
   bool
   Skin::Load (Cache* cache, const String &name)
   {
      AutoReadStream file (name);
      ReadStream& str = file.Get();
      
      while (! str.fail())
      {
	  String mat;
	  std::getline (str, mat);

	  if (mat.empty())
	     continue;
	  
	  Material *pmat;
	  cache->Get (V_MATERIAL, mat, &pmat);
	  
	  if (pmat)
	     m_Materials.push_back (pmat);
      }

      return m_Materials.size() != 0;
   }


   /// Insert a new material in this skin.
   void
   Skin::Insert (Material *mat)
   {
      mat->Ref();
      m_Materials.push_back (mat);
   }

   /**
    * Give the integer identifier corresponding to the 
    * material name.
    */
   int
   Skin::Find (const Ark::String &name)
   {
      size_t i;

      for (i = 0; i != m_Materials.size(); i++)
      {
	 if (m_Materials[i]->Name() == name)
	 {
	    Sys()->Log ("%s index is %d\n", name.c_str(), i);
	    return i;
	 }
      }

      // FIXME: what should I return ??
      return 0;
   }

   /// Get the material whith the given identifier.
   Material*
   Skin::GetMaterial (int id)
   {
      return m_Materials[id];
   }
      
}
