/*
** Ark - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the ArkRPG Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef WIN32
 #include <windows.h>
#endif

#include <Client/MessageView.h>

namespace Client
{

   MessageLabel::MessageLabel () : m_Image (NULL)
   {}
   
   MessageLabel::MessageLabel (int xpos, int ypos,
			       Ark::Entity *my,
			       const Ark::String &msg,
			       Ark::Entity *dest) :
      m_Image (NULL),
      m_MyEntity (my),
      m_DestEntity (dest),
      m_X(xpos),
      m_Y(ypos)
   {
       Ark::String::size_type pos = msg.find("|", 0);

      if (pos != Ark::String::npos)
      {
	 m_Reply = msg.substr(0, pos);
	 m_Msg = msg.substr(pos + 1, Ark::String::npos);
      }
      else
      {
	 m_Reply = m_Msg = msg;
      }
   }

   MessageLabel::MessageLabel (int xpos, int ypos,
			       Ark::Texture *image) : 
      m_Msg (""),
      m_Image (image),
      m_MyEntity (NULL),
      m_DestEntity (NULL),
      m_X(xpos),
      m_Y(ypos)
   {
      m_Image->Ref();
   }

   MessageLabel::~MessageLabel()
   {
      if (m_Image)
	 m_Image->Unref();
   }
		       
   void
   MessageLabel::Render(UIRenderer *ui)
   {
      if (m_Image)
      {
	 int w = m_Image->m_Width;
	 int h = m_Image->m_Height;

	 ui->SetTexture (m_Image);
	 ui->DrawTexturedRectangle (m_X, m_Y, w, h);
      }
      else
      {
	 Ark::Font *font;
	 if (m_DestEntity)
	    font = ui->GetFont ("arial1-2-ffff00");
	 else
	    font = ui->GetFont ("arial1-2-ffffff");

	 ui->DrawString (font, m_Msg, m_X, m_Y);
      }
   }

   bool
   MessageLabel::HandleMButton (bool down, int b, int x, int y)
   {
      if (m_DestEntity == NULL || m_Image != NULL)
	 return false;

      Ark::Font *font = g_UI->GetFont ("arial1-2-ffff00");
      int maxs[2], mins[2];
      font->GetStringBounds (m_Msg, Ark::CENTER_LEFT, mins, maxs);

      maxs[0] += m_X; mins[0] += m_X;
      maxs[1] += m_Y; mins[1] += m_Y;

      if (x >= mins[0] && x <= maxs[0] &&
	  y >= mins[1] && y <= maxs[1])
      {
	 m_MyEntity->AddMessage (m_DestEntity, m_Reply);

	 // Remove message..
	 m_MyEntity->m_Messages.erase
	    (m_MyEntity->m_Messages.begin());

	 return true;
      }

      return false;
   }
   
   ////////////////////////////////////////////////////////////////////////


   MessageView::MessageView () : 
   	 m_PlayerEntity (0),
	 m_Image (0)
   {}

   MessageView::~MessageView ()
   { if (m_Image) m_Image->Unref();}

   void
   MessageView::UpdateChildren (UIRenderer *ui, Ark::Entity *ent)
   {
      if (!m_Image)
      {
	 ///FIXME: hardcoded image path
	 ui->Rdr()->GetCache().Get
	    (Ark::V_TEXTURE, "{icons}/chr-deichey.png", &m_Image);
      }

      int ypos = 20;
      int xskip = 5;
      int yskip = 5;

      if (m_Image)
      {
	 int w = m_Image->m_Width;
	 m_Children.push_back (MessageLabel (xskip, yskip, m_Image));

	 xskip += w;
      }
	    
      Ark::EntityMessage &msg = ent->m_Messages[0];

      Ark::Font *font = ui->GetFont ("arial1-2-ffffff");
      m_Children.push_back
	 (MessageLabel (xskip, ypos, ent, msg.m_Message));
	    
      font = ui->GetFont ("arial1-2-ffff00");
      for (size_t i = 0; i < msg.m_AnswerList.size(); i++)
      {
	 ypos += ui->GetStringHeight (font,msg.m_AnswerList[i]);
	       
	 m_Children.push_back
	    (MessageLabel (20+xskip, ypos, ent,
			   msg.m_AnswerList[i], msg.m_Entity));
      }	    
   }
   
   void
   MessageView::Render(UIRenderer *ui, Ark::Entity *ent)
   {
      if (ent == NULL || ent->m_Messages.empty())
	 return;

      if (m_Children.empty())
	 UpdateChildren (ui, ent);
	    
      // Render my children.
      std::vector<MessageLabel>::iterator i;
      for (i = m_Children.begin(); i != m_Children.end(); i++)
	 i->Render (ui);

      m_PlayerEntity = ent;
   }

   bool
   MessageView::HandleMButton (bool down, int b, int x, int y)
   {
      if (m_PlayerEntity == NULL ||
	  m_PlayerEntity->m_Messages.empty())
	 return false;

      if (down == false || b != 1)
	 return false;

	    
      std::vector<MessageLabel>::iterator i;
	    
      for (i = m_Children.begin(); i != m_Children.end(); i++)
      {
	 if (i->HandleMButton (down, b, x, y))
	 {
	    m_Children.resize(0);
	    return true;
	 }
      }

      return false;
   }

}

