/*
** ArkRPG - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the ArkRPG Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifdef HAVE_CONFIG_H
 #include <config.h>
#endif


#ifdef WIN32
 #include <windows.h>
#endif

#include <Client/Client.h>
#include <Ark/ArkBuffer.h>

namespace Client
{

class Callback
{
public:
  int message;
  void (*function) (Ark::Packet *packet);
};

/*Callback s_Callbacks[] =
{
  { Shared::UPD_ENTITIES,          &Client::UpdEntities         },
  { Shared::UPD_MSG_ADDCHAT,       &Client::UpdMsgAddChat       },
  { Shared::UPD_MSG_ADDSERVER,     &Client::UpdMsgAddServer     },

  { Shared::UPD_ENTITY_ADD,        &Client::UpdEntityAdd        },
  { Shared::UPD_ENTITY_DESTROY,    &Client::UpdEntityDestroy    },
  { Shared::UPD_ENTITY_CLIENT,     &Client::UpdEntityClient     },

  { Shared::UPD_RESULT,            &Client::UpdResult           },
  { Shared::UPD_TERRAIN_SETCOORDS, &Client::UpdTerrainSetCoords },
  { 0, NULL }
};*/

void
Client::UpdResult (Ark::Packet *packet)
{
  int msg;
  int error;

  (*packet) >> msg >> error;

  if (error)
  {
    Ark::String str;

    (*packet) >> str;
    CbError (str.Get());
  }
  else if (msg == Shared::MSG_CONNECT)
    m_Logged = true;
}

void
Client::UpdTerrainSetCoords (Ark::Packet *packet)
{
  int x, y;

  (*packet) >> x >> y;
  CbLoadWorld (x, y);
}

void
Client::UpdMsgAddChat (Ark::Packet *packet)
{
  Ark::String who;
  Ark::String msg;

  (*packet) >> who >> msg;
  CbChatMsg (who, msg);
}

void
Client::UpdMsgAddServer (Ark::Packet *packet)
{
  Ark::String str;

  (*packet) >> str;
  CbServerMsg (str);
}

void
Client::UpdEntityAdd (Ark::Packet *packet)
{
  int id;
  (*packet) >> id;

  if (m_World)
  {
    // Create the entity
    Ark::Entity *ent = new Ark::Entity (GetWorld());
    ent->m_ID = id;

    // Add it to the world
    GetWorld ()->Add (ent);
  }
}

void
Client::UpdEntityDestroy (Ark::Packet *packet)
{
  int id;
  (*packet) >> id;

  if (m_World)
  {
    Ark::Entity *ent = m_World->Find (id);
    if (ent) ent->SetDead();
  }
}

void
Client::UpdEntityClient (Ark::Packet *packet)
{
  int id;
  (*packet) >> id;

  if (m_World)
  {
    m_Entity = m_World->Find (id);
  }
  else
  {
    Ark::Sys()->Log ("Update client entity\n"
                           "  error: world is nil\n");
    m_Entity = NULL;
  }
}

void
Client::UpdEntities (Ark::Packet *packet)
{
  int id;

  // FIXME: This won't work (and will corrupt the packet by not reading
  // all the entity data) if the world hasn't been created
  do
  {
    (*packet) >> id;

    if (m_World)
    {
      Ark::Entity *ent = m_World->Find (id);
      if (ent)
        ent->Read (*packet);
    }
  } while (!packet->Eof () && id > 0);
}

void
Client::HandleUpdate (Ark::Packet *packet)
{
  int m;

  while (packet->Buf()->Size()>=4 && !packet->Eof())
  {
    (*packet) >> m;
    bool found = false;

    for (int i = 0; s_Callbacks[i].message; i++)
    {
      if (s_Callbacks[i].message == m)
      {
        (this->*(s_Callbacks[i].function)) (packet);
        found = true;
        break;
      }
    }

    if (!found)
      Ark::Sys()->Warning ("HandleUpdate\n"
                                 "  error: Unknown message ID: %x", m);
  }

  return;
}

}
