/*
** ArkRPG - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the ArkRPG Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef CLIENT_WINDOWWIDGET_H
#define CLIENT_WINDOWWIDGET_H

#include <Client/ContainerWidget.h>

namespace Client
{
    /**
     * This class represents a Window which can contain one
     * widget---which can be a Panel---
     * It has a title and a frame.
     */
    class WindowWidget : public ContainerWidget
    {
    protected:
	/** The title of the window */
	ArkString m_Title;
	
	/** The position of the window */
	int m_X, m_Y;

	/** The sizes of the window */
	int m_Height, m_Width;

    public:
	/**
	 * Constructor with the title
	 * \param title An ArkString containing the title of the window
	 */
	WindowWidget(const ArkString& title = "")
	{
	    this->m_Title = title;
	    this->m_X = 0;
	    this->m_Y = 0;
	    this->m_Height = 0;
	    this->m_Width = 0;
	}

	/**
	 * Sets the title of the window
	 * \param title An ArkString containing the title of the window
	 */
	void setTitle(const ArkString& title)
	{
	    this->m_Title = title;
	}

	/**
	 * Returns the title of the window
	 * \return The title of the window in an ArkString.
	 */
	// It doesn't return a reference not to make the title modifiable
	// outside setTitle()
	ArkString getTitle()
	{
	    return this->m_Title;
	}

	/**
	 * Sets the position of the Window
	 * \param x x position of the Window
	 * \param y y position of the Window
	 */
	void setPosition(int x, int y)
	{
	    this->m_X = x;
	    this->m_Y = y;
	}

	/**
	 * Returns the X position of the widget
	 * \return The X position of the widget
	 */
	int getXPosition()
	{
	    return this->m_X;
	}

	/**
	 * Returns the Y position of the widget
	 * \return The Y position of the widget
	 */
	int getYPosition()
	{
	    return this->m_Y;
	}

	/**
	 * Sets the sizes of the window
	 * \param height The height of the Window
	 * \param width The width of the Window
	 */
	void setSizes(int height, int width)
	{
	    this->m_Height = height;
	    this->m_Width = width;
	}

	/**
	 * Returns the height of the window
	 * \return The height of the window
	 */
	int getHeight()
	{
	    return this->m_Height;
	}

	/**
	 * Returns the width of the window
	 * \return The width of the window
	 */
	int getWidth()
	{
	    return this->m_Width;
	}
    }
}

#endif
